# -*- coding: utf-8 -*-
from django.db import models
from django.contrib.auth.models import User
import datetime
from decimal import Decimal

# Create your models here.
class BuyableType(models.Model):
	name = models.CharField(max_length=100)
	
	def __unicode__(self):
		return self.name


class Buyable(models.Model):
	name = models.CharField(max_length=100)
	price = models.DecimalField(max_digits=8, decimal_places=2)
	image = models.ImageField(upload_to='img/buyable/')
	deposit = models.DecimalField(max_digits=8, decimal_places=2)
	description = models.TextField()
	buyableType = models.ManyToManyField(BuyableType)

	def hasDeposit(self):
		return self.deposit > Decimal(0)
	
	def createPurchase(self, isDeposit=False):
		p = Purchase()
		if isDeposit:
			p.price = self.deposit
		else:
			p.price = self.price
		p.dateTime = datetime.datetime.now()
		p.buyable = self
		
		return p
	
	def __unicode__(self):
		item = "%s (%.2f EUR" % (self.name, self.price)
		if self.hasDeposit():
			item += "/%.2f Pfand" % self.deposit
		item += ")"
		return item

class Order(models.Model):
	user = models.ForeignKey(User)
	price = models.DecimalField(max_digits=8, decimal_places=2)
	dateTime = models.DateTimeField()

	def create(self, user=None):
		models.Model.__init__(self)
		self.price = Decimal(0)
		self.dateTime = datetime.datetime.now()
		if user:
			self.user = user
	
	def addItems(self, items):
		for item in items:
			self.purchase.add(item)
			self.price += item.price
	
	def updatePrice(self, commit=False, updateBalance=False):
		self.price = Decimal(0)
		for item in self.purchase_set.all():
			self.price += item.price
		if commit or updateBalance:
			# TROLL MODE ON!
			profile = self.user.get_profile()
			profile.balance -= self.price
			profile.save()
	
	def __unicode__(self):
		return "Price %s, User %s" % (self.price, self.user)
	
	#def save(self, *args, **kwargs):
		#profile = self.user.get_profile()
		#if self.id == None:
			## new item, get it!
			#profile.balance -= self.price
			#profile.save()
		#else:
			## get old 
		#super(Order, self).save(*args, **kwargs)


class Purchase(models.Model):
	order = models.ForeignKey(Order)
	price = models.DecimalField(max_digits=8, decimal_places=2)
	isDeposit = models.BooleanField()
	buyable = models.ForeignKey(Buyable)

	@staticmethod
	def create(order, buyable, isDeposit=False):
		p = Purchase()
		p.order = order
		p.isDeposit = isDeposit
		if p.isDeposit:
			p.price = buyable.deposit
		else:
			p.price = buyable.price
		p.dateTime = datetime.datetime.now()
		p.buyable = buyable
		
		return p

	def __unicode__(self):
		return "%s%s" % (self.buyable.name, (self.isDeposit and " (deposit)" or ""))
	
#	def save(self, *args, **kwargs):
#		profile = self.user.get_profile()
#		if self.id == None:
#			# new item, get it!
#			profile.balance -= self.price
#			profile.save()
#		super(Purchase, self).save(*args, **kwargs)


